import { LanguageSupport, LRLanguage } from '@codemirror/language';
import { LintSource, linter } from '@codemirror/lint';
import { LRParser } from '@lezer/lr';
import { Completion, CompletionContext } from '@codemirror/autocomplete';

declare const parser: LRParser;

declare const builtInFunctions: readonly ["acos", "acosh", "asin", "asinh", "atan", "atan2", "atanh", "cbrt", "cos", "cosh", "erf", "erfc", "exp", "frexp", "gamma", "hypot", "ldexp", "lgamma", "log", "log10", "log2", "sin", "sinh", "sqrt", "tan", "tanh", "if", "and", "or", "xor", "not", "switch", "min", "max", "sum", "avg", "count", "round", "roundup", "rounddown", "abs", "case", "left", "right", "mid", "len", "substitute", "concat", "contains", "map", "filter", "all", "any", "pluck"];

interface DentakuLanguageCompletionOptions {
    /**
     * CodeMirror `Completion` objects for known variables.
     *
     * @example
     * ["a", "b"].map(name => ({ label: name, type: "variable" }))
     */
    variableEntries?: Array<Completion>;
    /**
     * Converter of built-in Dentaku functions into `Completion` objects.
     * Allows you to add additional information to completions like descriptions.
     */
    makeEntryForBuiltIns?: (name: (typeof builtInFunctions)[number]) => Completion | null | false;
}
declare function dentakuCompletions({ variableEntries, makeEntryForBuiltIns, }?: DentakuLanguageCompletionOptions): (context: CompletionContext) => {
    options: Completion[];
    from: number;
    validFor: RegExp;
} | null;

/** Message codes that can be overridden with custom messages. */
interface ErrorMessages {
    /** Generic "invalid syntax" message when no better message can be inferred. */
    invalidSyntax: string;
    /** Missing a closing bracket in a list of function arguments. */
    closingBracketMissing: string;
    /** Expected a comma before this function argument. */
    expectedCommaBefore: string;
    /** Expected an operator before this expression. */
    expectedOperatorBefore: string;
    /** The function name needs parentheses to be called. */
    callParenthesesMissing: string;
    /** This variable is not defined. */
    undefinedVariable: string;
    /** Expected at least one parameter in a function call. */
    expectedAtLeastOneParameter: string;
    /** Expected that many parameters, found this many. */
    parameterCountMismatch: (expectedAmount: number, actualAmount: number) => string;
}
interface DentakuLinterOptions {
    /** List of variable names to not treat as undefined. */
    knownVariables?: Array<string>;
    /** Custom error messages. */
    messages?: ErrorMessages;
}
declare function dentakuLinter({ knownVariables, messages, }?: DentakuLinterOptions): LintSource;
/** Special character reserved for invalid syntax trees. */
declare const invalidSyntax = "\u26A0";

/**
 * The full Dentaku language support, including autocompletion and linting.
 *
 * @example
 * const codeMirrorExtensions = [
 *   ...otherExtensions,
 *   dentaku(),
 * ];
 */
declare function dentaku({ linterOptions: { codeMirrorConfig, ...linterOptions }, completionOptions, }: {
    linterOptions: {
        codeMirrorConfig?: Parameters<typeof linter>[1];
    } & DentakuLinterOptions;
    completionOptions: DentakuLanguageCompletionOptions;
}): LanguageSupport;
/**
 * Only the parser and syntax highlighter for Dentaku.
 *
 * Use this only if you want to omit/overwrite either linting or autocompletion.
 */
declare const dentakuLanguage: LRLanguage;

export { DentakuLanguageCompletionOptions, DentakuLinterOptions, ErrorMessages, dentaku, dentakuCompletions, dentakuLanguage, dentakuLinter, parser as dentakuParser, invalidSyntax };
